﻿#ifndef CAMERA_H
#define CAMERA_H

#include <windows.h>
#include <xnamath.h>

/// <summary>
/// Orbiting style camera
/// </summary>
class Camera
{
  public:
    /// <summary>
    /// Creates a camera based on the specified data
    /// </summary>
    /// <param name="aspect_ratio">
    /// Aspect ratio for the camera
    /// </param>
    /// <param name="lookat">
    /// Where the camera should be looking in world space
    /// </param>
    /// <param name="range">
    /// distance the camera's position should be from the look at
    /// </param>
    /// <param name="yaw">
    /// angle of the current position of the orbit, in radians in the range
    /// [-pi,pi]
    /// </param>
    /// <param name="pitch">
    /// angle to pitch the camera up/down by, in radians in the range
    /// [-pi/2,pi/2]
    /// </param>
    Camera(float aspect_ratio,XMFLOAT4 lookat,float range,float yaw,
      float pitch);;
    
    /// <summary>
    /// Sets the x coordinate in world space of what the camera should be
    /// looking at
    /// </summary>
    /// <param name="x">
    /// x coordinate in world space
    /// </param>
    void SetLookAtX(float x);
    
    /// <summary>
    /// Sets the y coordinate in world space of what the camera should be
    /// looking at
    /// </summary>
    /// <param name="y">
    /// x coordinate in world space
    /// </param>
    void SetLookAtY(float y);
    
    /// <summary>
    /// Sets the z coordinate in world space of what the camera should be
    /// looking at
    /// </summary>
    /// <param name="z">
    /// x coordinate in world space
    /// </param>
    void SetLookAtZ(float z);
    
    /// <summary>
    /// Sets the distance the camera's position should be from the look at
    /// </summary>
    /// <param name="range">
    /// distance the camera's position should be from the look at
    /// </param>
    void SetOrbitRange(float range);
    
    /// <summary>
    /// Sets the yaw for determining the camera's position relative to the look
    /// at
    /// </summary>
    /// <param name="yaw">
    /// angle of the current position of the orbit, in radians in the range
    /// [-pi,pi]
    /// </param>
    void SetOrbitYaw(float yaw);
    
    /// <summary>
    /// Sets the pitch for determining the camera's position relative to the
    /// look at
    /// </summary>
    /// <param name="pitch">
    /// angle to pitch the camera up/down by, in radians in the range
    /// [-pi/2,pi/2]
    /// </param>
    void SetOrbitPitch(float pitch);
    
    /// <summary>
    /// Retrieves the current projection matrix
    /// </summary>
    /// <returns>
    /// projection matrix
    /// </returns>
    const XMFLOAT4X4& GetProjection() const;
    
    /// <summary>
    /// Retrieves the current view matrix
    /// </summary>
    /// <returns>
    /// view matrix
    /// </returns>
    const XMFLOAT4X4& GetView() const;
    
    /// <summary>
    /// Retrieves the right unit vector for the camera
    /// </summary>
    /// <returns>
    /// right vector
    /// </returns>
    const XMFLOAT4& GetRight() const;
    
    /// <summary>
    /// Retrieves the up unit vector for the camera
    /// </summary>
    /// <returns>
    /// up vector
    /// </returns>
    const XMFLOAT4& GetUp() const;
  
  private:
    /// <summary>
    /// Causes the calculated fields to have their values recalculated based on
    /// the current non-calculated fields
    /// </summary>
    void Recalc();

    /// <summary>
    /// View matrix
    /// </summary>
    XMFLOAT4X4 m_view;

    /// <summary>
    /// Projection matrix
    /// </summary>
    XMFLOAT4X4 m_proj;
    
    /// <summary>
    /// Calculated position of the camera
    /// </summary>
    XMFLOAT4 m_pos;
    
    /// <summary>
    /// Calculated unit vector for direction the camera is pointing
    /// </summary>
    XMFLOAT4 m_dir;
    
    /// <summary>
    /// Calculated unit vector for the up direction of the camera
    /// </summary>
    XMFLOAT4 m_up;
    
    /// <summary>
    /// Calculated unit vector that points to the right of the direction the
    /// camera is facing
    /// <remarks>
    /// Useful for billboarding
    /// </remarks>
    /// </summary>
    XMFLOAT4 m_right;

    /// <summary>
    /// Where the camera should be looking in world space
    /// </summary>
    XMFLOAT4 m_look_at;
    
    /// <summary>
    /// distance the camera's position should be from the look at
    /// </summary>
    float m_range;
    
    /// <summary>
    /// angle of the current position of the orbit, in radians in the range
    /// [-pi,pi]
    /// </summary>
    float m_yaw;
    
    /// <summary>
    /// angle to pitch the camera up/down by, in radians in the range
    /// [-pi/2,pi/2]
    /// </summary>
    float m_pitch;
};
#endif /* CAMERA_H */
