#include "GameMain.h"
#include "VertexDeclarations/Vertex_PositionTexture.h"
#include "log.h"
#include "math_extension.h"
#include "Messages/Message.h"
#include "Messages/MsgCloseConnection.h"
#include "Messages/MsgConnectionRejected.h"
#include "Messages/MsgSetSingle.h"
#include "Messages/MsgSetString.h"
#include "Messages/MsgColorSet.h"
#include "Messages/MsgColorSeq.h"
#include "ParticleSystem/DispersionControl/Directional.h"
#include "ParticleSystem/DispersionControl/PartialSphere.h"
#include "ParticleSystem/DispersionControl/Spherical.h"
#include "ParticleSystem/EmitterShapes/Circle.h"
#include "ParticleSystem/EmitterShapes/Hemisphere.h"
#include "ParticleSystem/EmitterShapes/Line.h"
#include "ParticleSystem/EmitterShapes/Point.h"
#include "ParticleSystem/EmitterShapes/Ring.h"
#include "ParticleSystem/EmitterShapes/Sphere.h"
#include "ParticleSystem/EmitterShapes/Square.h"
#include "ParticleSystem/EmitterShapes/SquarePerimeter.h"
using namespace std;

GameMain::GameMain(WCHAR* title,HINSTANCE instance)
:Game(title,instance),
 m_client(NULL),
 m_cam(NULL)
{
  // start the server
  ServerSocket::Init();
  m_server = ServerSocket::Create("2048");
  if (m_server == NULL)
  {
    log_print("Error: failed to start the server");
    exit(1);
  }
}

GameMain::~GameMain()
{
  if (m_client)
  {
    MsgCloseConnection msg;
    msg.Send(*m_client);
  }
  
  delete m_cam;
  delete m_client;
  delete m_server;
  ServerSocket::Cleanup();
}

void GameMain::LoadContent()
{
  ParticleSystem::LoadContent(*m_graphics,*m_content);
  m_particle = new ParticleSystem(*m_graphics,*m_content);
  
  m_emitters.insert(make_pair(EMITTER_CIRCLE,new Circle));
  m_emitters.insert(make_pair(EMITTER_HEMISPHERE,new Hemisphere));
  m_emitters.insert(make_pair(EMITTER_LINE,new Line));
  m_emitters.insert(make_pair(EMITTER_POINT,new Point));
  m_emitters.insert(make_pair(EMITTER_RING,new Ring));
  m_emitters.insert(make_pair(EMITTER_SPHERE,new Sphere));
  m_emitters.insert(make_pair(EMITTER_SQUARE,new Square));
  m_emitters.insert(make_pair(EMITTER_SQUARE_PERIMETER,new SquarePerimeter));
  
  D3D11_VIEWPORT viewport = m_graphics->GetViewport(0);
  XMFLOAT4 lookat(0,0,0,1);
  m_cam = new Camera(viewport.Width / viewport.Height,lookat,10,0,0);
}

void GameMain::UnloadContent()
{
  delete m_particle;
  ParticleSystem::UnloadContent(*m_graphics,*m_content);
}

void GameMain::Update(UINT ms)
{
  // check if a client is trying to connect
  if (m_server->PendingClient())
  {
    // only allow a client connection if there isn't one already
    if (m_client == NULL)
    {
      m_client = m_server->Accept();
    }
    else
    {
      // send back an error message
      TCPSocket* sock = m_server->Accept();
      MsgConnectionRejected msg;
      msg.Send(*m_client);
      delete sock;
    }
  }
  
  // check if there are any messages from the client
  while (m_client != NULL && m_client->DataPending())
  {
    Message* msg = Message::Receive(*m_client);
    switch (msg->GetMessageID())
    {
      case MSG_CLOSE_CONNECTION:
        delete m_client;
        m_client = NULL;
        break;
        
      case MSG_SET_SINGLE:
        {
          MsgSetSingle* tmp = (MsgSetSingle*)msg;
          SetParam(tmp->GetParamID(),tmp->GetValue());
        }
        break;
        
      case MSG_SET_STRING:
        {
          MsgSetString* tmp = (MsgSetString*)msg;
          SetParam(tmp->GetParamID(),tmp->GetValue());
        }
        break;
        
      case MSG_COLOR_SET:
        SetParam(((MsgColorSet*)msg)->GetValue());
        break;
        
      case MSG_COLOR_SEQ:
        SetParam(((MsgColorSeq*)msg)->GetValue());
        break;
        
      default:
        // ignore
        break;
    }
    delete msg;
  }
  
  // update the particle system
  m_particle->Update(ms,m_graphics->GetImmediateContext());
}

void GameMain::Draw(UINT ms)
{
  // only using 1 d3d11 thread, so just get the immediate context
  ID3D11DeviceContext* context = m_graphics->GetImmediateContext();
  
  // draw the particle system
  m_particle->Draw(*m_cam,context);
}

void GameMain::SetParam(ParameterIDs id,float val)
{
  switch (id)
  {
    case PARAM_CAM_LOOKAT_X:
      m_cam->SetLookAtX(val);
      break;
      
    case PARAM_CAM_LOOKAT_Y:
      m_cam->SetLookAtY(val);
      break;
      
    case PARAM_CAM_LOOKAT_Z:
      m_cam->SetLookAtZ(val);
      break;
      
    case PARAM_CAM_ORBIT_RANGE:
      m_cam->SetOrbitRange(val);
      break;
      
    case PARAM_CAM_ORBIT_YAW:
      m_cam->SetOrbitYaw(to_radians(val));
      break;
      
    case PARAM_CAM_ORBIT_PITCH:
      m_cam->SetOrbitPitch(to_radians(val));
      break;
      
    case PARAM_EMITTER_SHAPE:
      {
        map<EmitterShapes,EmitterShapeBase*>::iterator emitter_it =
          m_emitters.find((EmitterShapes)(int)val);
        if (emitter_it != m_emitters.end())
        {
          m_particle->GetEmitterSettings().shape = emitter_it->second;
        }
      }
      break;
      
    case PARAM_EMITTER_POS_X:
      m_particle->GetEmitterSettings().pos.x = val;
      break;
      
    case PARAM_EMITTER_POS_Y:
      m_particle->GetEmitterSettings().pos.y = val;
      break;
      
    case PARAM_EMITTER_POS_Z:
      m_particle->GetEmitterSettings().pos.z = val;
      break;
      
    case PARAM_EMITTER_ROT_YAW:
      m_particle->GetEmitterSettings().yaw = to_radians(val);
      break;
      
    case PARAM_EMITTER_ROT_PITCH:
      m_particle->GetEmitterSettings().pitch = to_radians(val);
      break;
      
    case PARAM_EMITTER_ROT_ROLL:
      m_particle->GetEmitterSettings().roll = to_radians(val);
      break;
      
    case PARAM_EMITTER_SCALE_X:
      m_particle->GetEmitterSettings().scale.x = val;
      break;
      
    case PARAM_EMITTER_SCALE_Y:
      m_particle->GetEmitterSettings().scale.y = val;
      break;
      
    case PARAM_EMITTER_SCALE_Z:
      m_particle->GetEmitterSettings().scale.z = val;
      break;
      
    case PARAM_EMITTER_SPAWN_TIME:
      m_particle->GetEmitterSettings().spawn_time = (UINT)(val * 1000);
      break;
      
    case PARAM_EMITTER_SPAWN_RATE:
      m_particle->GetEmitterSettings().spawn_rate = val / 1000;
      m_particle->UpdateMaxParticles(m_mspf);
      break;
      
    case PARAM_PARTCILE_DISPERSION_TYPE:
      m_particle->SetActiveDispersion((ParticleDispersion)(int)val);
      break;
      
    case PARAM_PARTICLE_DISPERSION_DIR_X:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_DIRECTIONAL);
        if (disp_it != disps.end())
        {
          Directional* dir = (Directional*)disp_it->second;
          dir->GetRawVector().x = val;
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_DIR_Y:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_DIRECTIONAL);
        if (disp_it != disps.end())
        {
          Directional* dir = (Directional*)disp_it->second;
          dir->GetRawVector().y = val;
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_DIR_Z:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_DIRECTIONAL);
        if (disp_it != disps.end())
        {
          Directional* dir = (Directional*)disp_it->second;
          dir->GetRawVector().z = val;
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_PAR_SPHERE_MIN_X:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_PARTIAL_SPHERE);
        if (disp_it != disps.end())
        {
          PartialSphere* sphere = (PartialSphere*)disp_it->second;
          sphere->SetMinX(to_radians(val));
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_PAR_SPHERE_MAX_X:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_PARTIAL_SPHERE);
        if (disp_it != disps.end())
        {
          PartialSphere* sphere = (PartialSphere*)disp_it->second;
          sphere->SetMaxX(to_radians(val));
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_PAR_SPHERE_MIN_Y:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_PARTIAL_SPHERE);
        if (disp_it != disps.end())
        {
          PartialSphere* sphere = (PartialSphere*)disp_it->second;
          sphere->SetMinY(to_radians(val));
        }
      }
      break;
      
    case PARAM_PARTICLE_DISPERSION_PAR_SPHERE_MAX_Y:
      {
        map<ParticleDispersion,DispersionBase*>& disps =
          m_particle->GetDispersions();
        map<ParticleDispersion,DispersionBase*>::iterator disp_it =
          disps.find(DISPERSION_PARTIAL_SPHERE);
        if (disp_it != disps.end())
        {
          PartialSphere* sphere = (PartialSphere*)disp_it->second;
          sphere->SetMaxY(to_radians(val));
        }
      }
      break;
      
    case PARAM_PARTICLE_SHAPE:
      m_particle->GetParticleSettings().shape = (ParticleShapes)(int)val;
      m_particle->UpdateParticleShape();
      break;
      
    case PARAM_PARTICLE_LIFETIME:
      m_particle->GetParticleSettings().lifetime = (UINT)(val * 1000);
      m_particle->UpdateMaxParticles(m_mspf);
      break;
      
    case PARAM_PARTICLE_MIN_SPEED:
      m_particle->GetParticleSettings().min_speed = val;
      break;
      
    case PARAM_PARTICLE_MAX_SPEED:
      m_particle->GetParticleSettings().max_speed = val;
      break;
      
    case PARAM_PARTICLE_SIZE:
      m_particle->GetParticleSettings().size = val;
      break;
      
    case PARAM_PARTICLE_ACC_X:
      m_particle->GetParticleSettings().acceleration.x = val;
      break;
      
    case PARAM_PARTICLE_ACC_Y:
      m_particle->GetParticleSettings().acceleration.y = val;
      break;
      
    case PARAM_PARTICLE_ACC_Z:
      m_particle->GetParticleSettings().acceleration.z = val;
      break;
      
    case PARAM_COLORING_MODE:
      m_particle->GetColoringSettings().active_mode =
        (ColoringMode)(int)(val + .5f);
      break;
      
    case PARAM_COLORING_SEQ_INTERPOLATE:
      m_particle->GetColoringSettings().interpolate = val > .5f;
      break;
      
    default:
      // ignore
      break;
  }
}

void GameMain::SetParam(ParameterIDs id,const string& val)
{
  if (id == PARAM_COLORING_TEXTURE_PATH)
  {
    if (val.size() > 0)
    {
      // send the data to the particle system
      ColoringSettings& settings = m_particle->GetColoringSettings();
      delete settings.texture;
      settings.texture = m_content->LoadTexture(val.c_str());
    }
    else
    {
      // clear the texture from the particle system
      ColoringSettings& settings = m_particle->GetColoringSettings();
      delete settings.texture;
      settings.texture = NULL;
    }
  }
  // ignore other ids
}

void GameMain::SetParam(vector<ColorSetEntry>& val)
{
  // make sure the entries have a total probability of 100%
  vector<ColorSetEntry>::iterator entry_it = val.begin();
  float total = 0;
  while (entry_it != val.end())
  {
    total += entry_it->GetChance();
    
    ++entry_it;
  }
  if (total > 0)
  {
    entry_it = val.begin();
    while (entry_it != val.end())
    {
      entry_it->SetChance(entry_it->GetChance() / total);
      
      ++entry_it;
    }
  }
      
  // send the data to the particle system
  m_particle->GetColoringSettings().color_set = val;
}

void GameMain::SetParam(vector<ColorSeqEntry>& val)
{
  // make sure the entries have a total probability of 100%
  vector<ColorSeqEntry>::iterator entry_it = val.begin();
  float total = 0;
  while (entry_it != val.end())
  {
    total += entry_it->GetLife();
    
    ++entry_it;
  }
  if (total > 0)
  {
    entry_it = val.begin();
    while (entry_it != val.end())
    {
      entry_it->SetLife(entry_it->GetLife() / total);
      
      ++entry_it;
    }
  }
      
  // send the data to the particle system
  m_particle->GetColoringSettings().color_seq = val;
}
