#ifndef COLOR_SEQ_ENTRY_H
#define COLOR_SEQ_ENTRY_H

#include <windows.h>
#include <xnamath.h>

/// <summary>
/// Contains data about a single color entry in a color sequence
/// </summary>
class ColorSeqEntry
{
  public:
    /// <summary>
    /// Retreives the number of bytes each entry takes up in a network message
    /// </summary>
    /// <returns>
    /// number of bytes each entry takes up in a network message
    /// </returns>
    static int BytesInMessage();
    
    /// <summary>
    /// Creates a color sequence entry from the specified byte array
    /// that contains the data portion of a received network message
    /// </summary>
    /// <param name="data">
    /// data portion of the message
    /// </param>
    /// <param name="offset">
    /// starting index of data to start extracting a entry.  This will be
    /// updated to account for the amount read by this function
    /// </param>
    ColorSeqEntry(char* data,int& offset);
    
    /// <summary>
    /// Creates a new color sequence entry
    /// </summary>
    /// <param name="color">
    /// Color to use for the entry
    /// </param>
    /// <param name="life">
    /// Percent of a particle's life, in the range (0,1], that the color
    /// should be used to color a particle
    /// </param>
    ColorSeqEntry(XMFLOAT3 color,float life);
    
    ~ColorSeqEntry();
    
    /// <summary>
    /// Sets the percent of the particle's life it should use this entry for
    /// </summary>
    /// <param name="life">
    /// Percent of a particle's life, in the range (0,1], that the color
    /// should be used to color a particle
    /// </param>
    void SetLife(float life);
    
    /// <summary>
    /// Retrieves the color for the entry
    /// </summary>
    /// <returns>
    /// Color for this entry
    /// </returns>
    const XMFLOAT3& GetColor() const;
    
    /// <summary>
    /// Retrieves the percent of the particle's life it should use this entry
    /// for
    /// </summary>
    /// <returns>
    /// Percent of a particle's life, in the range [0,1], that the color
    /// should be used to color a particle
    /// </returns>
    float GetLife() const;
    
  private:
    // disabled
    ColorSeqEntry();
    
    /// <summary>
    /// Color of the entry
    /// </summary>
    XMFLOAT3 m_color;
    
    /// <summary>
    /// Percent of a particle's life, in the range [0,1], that the color
    /// should be used to color a particle
    /// </summary>
    float m_life;
};

#endif /* COLOR_SEQ_ENTRY_H */
