#ifndef COLOR_SET_ENTRY_H
#define COLOR_SET_ENTRY_H

#include <windows.h>
#include <xnamath.h>

/// <summary>
/// Contains data about a single color entry in a color set
/// </summary>
class ColorSetEntry
{
  public:
    /// <summary>
    /// Retreives the number of bytes each entry takes up in a network message
    /// </summary>
    /// <returns>
    /// number of bytes each entry takes up in a network message
    /// </returns>
    static int BytesInMessage();
    
    /// <summary>
    /// Creates a color set entry from the specified byte array
    /// that contains the data portion of a received network message
    /// </summary>
    /// <param name="data">
    /// data portion of the message
    /// </param>
    /// <param name="offset">
    /// starting index of data to start extracting a entry.  This will be
    /// updated to account for the amount read by this function
    /// </param>
    ColorSetEntry(char* data,int& offset);
    
    /// <summary>
    /// Creates a new color set entry
    /// </summary>
    /// <param name="color">
    /// Color to use for the entry
    /// </param>
    /// <param name="prob">
    /// Percent chance, in the range [0,1], of the color being selected for
    /// the color set
    /// </param>
    ColorSetEntry(XMFLOAT3 color,float prob);
    
    ~ColorSetEntry();
    
    /// <summary>
    /// Sets the chance of this color being selected
    /// </summary>
    /// <param name="prob">
    /// Percent chance, in the range [0,1], of the color being selected for
    /// the color set
    /// </param>
    void SetChance(float prob);
    
    /// <summary>
    /// Retrieves the color for the entry
    /// </summary>
    /// <returns>
    /// Color for this entry
    /// </returns>
    const XMFLOAT3& GetColor() const;
    
    /// <summary>
    /// Retrieves the chance of this color being selected
    /// </summary>
    /// <returns>
    /// Percent chance, in the range [0,1], of the color being selected for
    /// the color set
    /// </returns>
    float GetChance() const;
    
  private:
    // disabled
    ColorSetEntry();
    
    /// <summary>
    /// Color of the entry
    /// </summary>
    XMFLOAT3 m_color;
    
    /// <summary>
    /// Percent chance, in the range [0,1], of the color being selected for
    /// the color set
    /// </summary>
    float m_chance;
};

#endif /* COLOR_SET_ENTRY_H */
