#ifndef COLORING_SETTINGS_H
#define COLORING_SETTINGS_H

#include <windows.h>
#include <xnamath.h>
#include <vector>
#include "Texture.h"
#include "ColoringMode.h"
#include "ColorSetEntry.h"
#include "ColorSeqEntry.h"

/// <summary>
/// Stores settings about the various coloring modes
/// </summary>
struct ColoringSettings
{
  /// <summary>
  /// Which coloring mode is currently in use
  /// </summary>
  ColoringMode active_mode;
  
  /// <summary>
  /// For the texture coloring mode, the texture to apply to the particles
  /// </summary>
  Texture* texture;
  
  /// <summary>
  /// For the color set coloring mode, the list of colors and their
  /// associated probabilities
  /// </summary>
  std::vector<ColorSetEntry> color_set;

  /// <summary>
  /// For color sequence coloring mode, the list of colors in order and the
  /// associated percentage of particle life that entry should be used for
  /// </summary>
  std::vector<ColorSeqEntry> color_seq;
  
  /// <summary>
  /// For the color sequence mode, true if colors should be interpolated
  /// between the current and next color, false if it should just be the
  /// current color
  /// </summary>
  bool interpolate;
};

/// <summary>
/// Determines the color of a particle at its creation time
/// </summary>
/// <param name="color">
/// color setting to use for determining initial particle color
/// </param>
/// <returns>
/// the color for the particle
/// </returns>
XMFLOAT3 GetInitialColor(const ColoringSettings& color);

/// <summary>
/// Determines the color of a particle at the specified percentage of its
/// life
/// <remarks>
/// Should only be called when color sequence is the active coloring mode
/// </remarks>
/// </summary>
/// <param name="color">
/// color setting to use for determining particle color
/// </param>
/// <param name="life">
/// how far the particle is in its life, in the range (0,1]
/// </param>
/// <returns>
/// color for the particle
/// </returns>
XMFLOAT3 GetColor(const ColoringSettings& color,float life);

#endif /* COLORING_SETTINGS_H */
