#include "ParticleSystem.h"
#include "log.h"
#include "DispersionControl/Directional.h"
#include "DispersionControl/PartialSphere.h"
#include "DispersionControl/Spherical.h"
#include "../GraphicsVector.h"
using namespace std;

VertexBuffer_PositionTexture* ParticleSystem::s_vert_tri         = NULL;
VertexBuffer_PositionTexture* ParticleSystem::s_vert_square      = NULL;
IndexBuffer16*                ParticleSystem::s_index_tri        = NULL;
IndexBuffer16*                ParticleSystem::s_index_square     = NULL;
VSPSEffect*                   ParticleSystem::s_billboard_effect = NULL;
TextureSampler*               ParticleSystem::s_sampler          = NULL;
BlendState*                   ParticleSystem::s_alpha_blend      = NULL;
DepthStencilState*            ParticleSystem::s_no_depth_test    = NULL;

void ParticleSystem::LoadContent(D3D11_Core& graphics,ContentManager& content)
{
  // load the effect for rendering
  D3D11_INPUT_ELEMENT_DESC layout[] =
  {
    // position
    { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0,
      D3D11_INPUT_PER_VERTEX_DATA, 0 },
    // uv
    { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12,
      D3D11_INPUT_PER_VERTEX_DATA, 0 },
    // instance pos
    { "TEXCOORD", 1, DXGI_FORMAT_R32G32B32A32_FLOAT, 1, 0,
      D3D11_INPUT_PER_INSTANCE_DATA, 1 },
    // instance color
    { "TEXCOORD", 2, DXGI_FORMAT_R32G32B32_FLOAT, 1, 16,
      D3D11_INPUT_PER_INSTANCE_DATA, 1 },
  };
  vector<UINT> buffer_widths;
  buffer_widths.push_back(sizeof(EffectConstBuffer));
  s_billboard_effect = content.LoadEffect(L"BillboardInstanceRenderer.fx",
    SM_5_0,layout,ARRAYSIZE(layout),buffer_widths);
  
  // create the triangle particle vertex and index buffer
  Vertex_PositionTexture verts[4] =
  {
    { XMFLOAT3(   0, .5f,0), XMFLOAT2(.5f,0) },
    { XMFLOAT3( .5f,-.5f,0), XMFLOAT2(  0,1) },
    { XMFLOAT3(-.5f,-.5f,0), XMFLOAT2(  1,1) },
    { XMFLOAT3(   0,   0,0), XMFLOAT2(  0,1) },
  };
  s_vert_tri = VertexBuffer_PositionTexture::Create(graphics,3,
    verts);
  if (s_vert_tri == NULL)
  {
    log_print("Failed to create triangle vertex buffer\n");
    exit(1);
  }
  WORD indices[4] = { 0, 1, 2, 0 };
  s_index_tri = IndexBuffer16::Create(graphics,3,indices);
  if (s_index_tri == NULL)
  {
    log_print("Failed to create triangle index buffer\n");
    exit(1);
  }
  
  // create the square particle vertex and index buffer
  verts[0].pos = XMFLOAT3(-.5f,-.5f,0);
  verts[1].pos = XMFLOAT3(-.5f, .5f,0);
  verts[2].pos = XMFLOAT3( .5f, .5f,0);
  verts[3].pos = XMFLOAT3( .5f,-.5f,0);
  verts[0].uv  = XMFLOAT2(1,1);
  verts[1].uv  = XMFLOAT2(1,0);
  verts[2].uv  = XMFLOAT2(0,0);
  verts[3].uv  = XMFLOAT2(0,1);
  s_vert_square = VertexBuffer_PositionTexture::Create(graphics,4,
    verts);
  if (s_vert_square == NULL)
  {
    log_print("Failed to create square vertex buffer\n");
    exit(1);
  }
  indices[0] = 0;
  indices[1] = 1;
  indices[2] = 3;
  indices[3] = 2;
  s_index_square = IndexBuffer16::Create(graphics,4,indices);
  if (s_index_square == NULL)
  {
    log_print("Failed to create square index buffer\n");
    exit(1);
  }
  
  // create the texture sampler
  D3D11_SAMPLER_DESC sample_desc;
  ZeroMemory(&sample_desc,sizeof(sample_desc));
  sample_desc.Filter         = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
  sample_desc.AddressU       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressV       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressW       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.ComparisonFunc = D3D11_COMPARISON_NEVER;
  sample_desc.MinLOD         = 0;
  sample_desc.MaxLOD         = D3D11_FLOAT32_MAX;
  s_sampler = TextureSampler::Create(sample_desc,graphics);
  if (s_sampler == NULL)
  {
    exit(1);
  }
  
  // create the alpha blend state
  D3D11_BLEND_DESC blend_desc;
  ZeroMemory(&blend_desc,sizeof(blend_desc));
	blend_desc.RenderTarget[0].BlendEnable           = TRUE;
  blend_desc.RenderTarget[0].SrcBlend              = D3D11_BLEND_SRC_ALPHA;
  blend_desc.RenderTarget[0].DestBlend             = D3D11_BLEND_INV_SRC_ALPHA;
  blend_desc.RenderTarget[0].BlendOp               = D3D11_BLEND_OP_ADD;
  blend_desc.RenderTarget[0].SrcBlendAlpha         = D3D11_BLEND_ONE;
  blend_desc.RenderTarget[0].DestBlendAlpha        = D3D11_BLEND_ZERO;
  blend_desc.RenderTarget[0].BlendOpAlpha          = D3D11_BLEND_OP_ADD;
  blend_desc.RenderTarget[0].RenderTargetWriteMask =
    D3D11_COLOR_WRITE_ENABLE_ALL;
  s_alpha_blend = BlendState::Create(blend_desc,graphics);
  
  // create the depth stencil state that disables depth testing
  D3D11_DEPTH_STENCIL_DESC depth_desc;
  ZeroMemory(&depth_desc,sizeof(depth_desc));
	depth_desc.DepthEnable                  = false;
	depth_desc.DepthWriteMask               = D3D11_DEPTH_WRITE_MASK_ALL;
	depth_desc.DepthFunc                    = D3D11_COMPARISON_LESS;
	depth_desc.StencilEnable                = true;
	depth_desc.StencilReadMask              = 0xff;
	depth_desc.StencilWriteMask             = 0xff;
	depth_desc.FrontFace.StencilFailOp      = D3D11_STENCIL_OP_KEEP;
	depth_desc.FrontFace.StencilDepthFailOp = D3D11_STENCIL_OP_INCR;
	depth_desc.FrontFace.StencilPassOp      = D3D11_STENCIL_OP_KEEP;
	depth_desc.FrontFace.StencilFunc        = D3D11_COMPARISON_ALWAYS;
	depth_desc.BackFace.StencilFailOp       = D3D11_STENCIL_OP_KEEP;
	depth_desc.BackFace.StencilDepthFailOp  = D3D11_STENCIL_OP_DECR;
	depth_desc.BackFace.StencilPassOp       = D3D11_STENCIL_OP_KEEP;
	depth_desc.BackFace.StencilFunc         = D3D11_COMPARISON_ALWAYS;
  s_no_depth_test = DepthStencilState::Create(depth_desc,graphics);
}

void ParticleSystem::UnloadContent(D3D11_Core& graphics,
  ContentManager& content)
{
  delete s_vert_tri;
  delete s_vert_square;
  delete s_index_tri;
  delete s_index_square;
  delete s_sampler;
  delete s_alpha_blend;
  delete s_no_depth_test;
  content.UnloadEffect(s_billboard_effect);
  
  s_vert_tri         = NULL;
  s_vert_square      = NULL;
  s_index_tri        = NULL;
  s_index_square     = NULL;
  s_billboard_effect = NULL;
  s_sampler          = NULL;
}

ParticleSystem::ParticleSystem(D3D11_Core& graphics,ContentManager& content)
:m_instance(NULL),
 m_graphics(graphics),
 m_content(content),
 m_particles(NULL),
 m_instance_data(NULL),
 m_additonal_spawn_time(0),
 m_runtime(0)
{
  memset(&m_particle_settings,0,sizeof(m_particle_settings));
  m_coloring_settings.active_mode = COLOR_TEXTURE;
  m_coloring_settings.texture     = NULL;
  m_coloring_settings.interpolate = false;
  m_active_dispersion             = DISPERSION_DIRECTIONAL;
  
  m_dispersions.insert(make_pair(DISPERSION_DIRECTIONAL,new Directional));
  m_dispersions.insert(make_pair(DISPERSION_PARTIAL_SPHERE,new PartialSphere));
  m_dispersions.insert(make_pair(DISPERSION_SPHERICAL,new Spherical));
  
  m_binding = new VertexBufferArray(2);
}

ParticleSystem::~ParticleSystem()
{
  map<ParticleDispersion,DispersionBase*>::iterator disp_it =
    m_dispersions.begin();
  while (disp_it != m_dispersions.end())
  {
    delete disp_it->second;
    
    ++disp_it;
  }
  
  m_content.UnloadTexture(m_coloring_settings.texture);
  
  delete m_particles;
  
  delete m_binding;
  
  delete m_instance;
  delete[]m_instance_data;
}

EmitterSettings& ParticleSystem::GetEmitterSettings()
{
  return m_emitter;
}

ParticleSettings& ParticleSystem::GetParticleSettings()
{
  return m_particle_settings;
}

ColoringSettings& ParticleSystem::GetColoringSettings()
{
  return m_coloring_settings;
}

void ParticleSystem::SetActiveDispersion(ParticleDispersion disp)
{
  m_active_dispersion = disp;
}

map<ParticleDispersion,DispersionBase*>& ParticleSystem::GetDispersions()
{
  return m_dispersions;
}

void ParticleSystem::UpdateMaxParticles(UINT mspf)
{
  m_additonal_spawn_time = 0;
  m_runtime = 0;
  
  if (m_emitter.spawn_rate > 0 && m_particle_settings.lifetime > 0)
  {
    // calculate the max number of particles that can exist at any given
    // time
    int max = (int)ceil(m_emitter.spawn_rate * m_particle_settings.lifetime);
    
    // adjust the max to handle the batching that occurs due to the update
    // interval
    int spawned_per_frame = (int)(m_emitter.spawn_rate * mspf);
    if (spawned_per_frame > 0)
    {
      // make max large enough to handle 1 addition frame of spawn
      max = (max / spawned_per_frame + 1) * spawned_per_frame;
    }
    
    // (re)create the ring buffer based on the size
    if (m_particles == NULL)
    {
      m_particles     = new RingBuffer<ParticleData>(max);
      m_instance_data = new ParticleInstance[max];
      m_instance      = InstanceBuffer::Create(m_graphics,max,
        sizeof(ParticleInstance),m_instance_data);
      m_binding->Set(1,*m_instance);
    }
    else if (m_particles->Size() != max)
    {
      delete m_instance;
      delete[]m_instance_data;
      
      RingBuffer<ParticleData>* tmp = m_particles;
      m_particles = new RingBuffer<ParticleData>(max,*tmp);
      delete tmp;
      
      m_instance_data = new ParticleInstance[max];
      m_instance      = InstanceBuffer::Create(m_graphics,max,
        sizeof(ParticleInstance),m_instance_data);
      m_binding->Set(1,*m_instance);
    }
  }
  else
  {
    delete m_particles;
    delete m_instance;
    delete m_instance_data;
    
    m_particles     = NULL;
    m_instance      = NULL;
    m_instance_data = NULL;
  }
}

void ParticleSystem::UpdateParticleShape()
{
  if (m_particle_settings.shape != SHAPE_TRIANGLE)
  {
    m_binding->Set(0,*s_vert_square);
  }
  else
  {
    m_binding->Set(0,*s_vert_tri);
  }
}

void ParticleSystem::Update(UINT ms,ID3D11DeviceContext* context)
{
  if (m_particles != NULL)
  {
    // update existing particles
    UpdateParticles(ms);
        
    // create new particles, if the emitter is still active
    if (m_runtime <= m_emitter.spawn_time)
    {
      CreateParticles(ms);
    }
        
    // update the instance buffer
    UpdateInstanceBuffer(context);
        
    // update how long the emitter has been running for
    m_runtime += ms;
        
    // if the emitter has been idle long enough, restart it
    if (m_runtime >=
      m_emitter.spawn_time + m_particle_settings.lifetime + 1000)
    {
      m_runtime = 0;
    }
  }
}

void ParticleSystem::Draw(const Camera& cam,ID3D11DeviceContext* context)
{
  if (m_particles != NULL && m_particles->GetInUse() > 0)
  {
    // save the old blend and depth stencil states
    BlendState*        old_blend = BlendState::GetCurrent(context);
    DepthStencilState* old_depth = DepthStencilState::GetCurrent(context);
    
    // update constant buffer
    EffectConstBuffer buffer;
    buffer.cam_right   = cam.GetRight();
    buffer.cam_up      = cam.GetUp();
    buffer.scale       = m_particle_settings.size;
    buffer.use_texture = m_coloring_settings.active_mode == COLOR_TEXTURE ?
      1 : 0;
    buffer.vp          = XMMatrixTranspose(
      XMLoadFloat4x4(&cam.GetView()) * XMLoadFloat4x4(&cam.GetProjection()));
    s_billboard_effect->SetConstantBuffer(context,0,&buffer);
    
    // set the blend and depth stencil states for alpha blending
    s_alpha_blend->MakeActive(context);
    s_no_depth_test->MakeActive(context);
    
    // apply the effect
    s_billboard_effect->MakeActive(context);
    
    // supply the texture data
    s_billboard_effect->UseTexture(context,m_coloring_settings.texture,0);
    s_billboard_effect->UseTextureSampler(context,s_sampler,0);
    
    // do the instance rendering
    m_binding->MakeActive(context);
    UINT verts = 0;
    if (m_particle_settings.shape == SHAPE_TRIANGLE)
    {
      s_index_tri->MakeActive(context);
      verts = s_vert_tri->GetNumEntries();
    }
    else
    {
      s_index_square->MakeActive(context);
      verts = s_vert_square->GetNumEntries();
    }
    m_graphics.DrawInstancedPrimitives(context,
      D3D11_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP,0,verts,0,
      m_particles->GetInUse());
    
    // restore the old blend and depth stencil states
    old_depth->MakeActive(context);
    old_blend->MakeActive(context);
    delete old_blend;
    delete old_depth;
  }
}

void ParticleSystem::CreateParticles(UINT ms)
{
  // determine how many particles to create
  int num = (int)(m_emitter.spawn_rate * (ms + m_additonal_spawn_time));

  if (num > 0)
  {
    m_additonal_spawn_time = 0;

    // create a matrix to convert emitter local space to world space
    XMMATRIX rot = 
      XMMatrixRotationRollPitchYaw(m_emitter.pitch,m_emitter.yaw,
      m_emitter.roll);
    XMMATRIX convert =
      XMMatrixScaling(m_emitter.scale.x,m_emitter.scale.y,m_emitter.scale.z)
      * rot
      * XMMatrixTranslation(m_emitter.pos.x,m_emitter.pos.y,m_emitter.pos.z);

    // create the particles
    for (int i = 0; i < num; ++i)
    {
      ParticleData part;
      float speed = (float)rand() / RAND_MAX
        * (m_particle_settings.max_speed - m_particle_settings.min_speed)
        + m_particle_settings.min_speed;
      part.color = GetInitialColor(m_coloring_settings);
      XMStoreFloat4(&part.position,
        XMVector4Transform(m_emitter.shape->SpawnPosition(),convert));
      XMStoreFloat4(&part.velocity,
        XMVector4Transform(
        m_dispersions[m_active_dispersion]->GetDirection() * speed,rot));
      part.existed_for = 0;

      m_particles->Add(part);
    }
  }
  else
  {
    m_additonal_spawn_time += ms;
  }
}

void ParticleSystem::UpdateParticles(UINT ms)
{
  // update the particles and put their data into the instance buffer
  float seconds = ms / 1000.0f;
  XMFLOAT4 acceleration = m_particle_settings.acceleration * seconds;
  int rem_cnt = 0;
  for (int i = 0; i < m_particles->GetInUse(); ++i)
  {
    // update how long the particle has existed for
    (*m_particles)[i].existed_for += ms;

    // if the particle is still alive, perform the update
    if ((*m_particles)[i].existed_for < m_particle_settings.lifetime)
    {
      // update the particle velocity based on acceleration
      (*m_particles)[i].velocity += acceleration;

      // update the particle position
      (*m_particles)[i].position += (*m_particles)[i].velocity * seconds;

      // if needed, update the color as well
      if (m_coloring_settings.active_mode == COLOR_SEQ)
      {
        float life = (*m_particles)[i].existed_for
          / (float)m_particle_settings.lifetime;
        (*m_particles)[i].color = GetColor(m_coloring_settings,life);
      }
    }
    // otherwise, discard the particle
    else
    {
      ++rem_cnt;
    }
  }
  m_particles->Free(rem_cnt);
}

void ParticleSystem::UpdateInstanceBuffer(ID3D11DeviceContext* context)
{
  for (int i = 0; i < m_particles->GetInUse(); ++i)
  {
    // put the data in the instance buffer
    m_instance_data[i].color    = (*m_particles)[i].color;
    m_instance_data[i].position = (*m_particles)[i].position;
  }
  m_instance->Update(context,m_instance_data);
}
