#ifndef PARTICLE_SYSTEM_H
#define PARTICLE_SYSTEM_H

#include <map>
#include "D3D11_Core.h"
#include "ContentManager.h"
#include "Effects/VSPSEffect.h"
#include "Buffers/VertexBuffer_PositionTexture.h"
#include "Buffers/IndexBuffer16.h"
#include "Buffers/InstanceBuffer.h"
#include "Buffers/VertexBufferArray.h"
#include "Texture.h"
#include "TextureSampler.h"
#include "BlendState.h"
#include "DepthStencilState.h"

#include "ParticleSettings.h"
#include "EmitterSettings.h"
#include "Coloring/ColoringSettings.h"
#include "DispersionControl/ParticleDispersion.h"
#include "DispersionControl/DispersionBase.h"
#include "../Camera.h"
#include "RingBuffer.h"

class ParticleSystem
{
  public:
    /// <summary>
    /// Allows the particle system type to load any content that it needs to
    /// <remarks>
    /// This must be called before any instances of ParticleSystem are created
    /// </remarks>
    /// </summary>
    /// <param name="graphics">
    /// graphics device that will be used by particle system instances
    /// </param>
    /// <param name="content">
    /// content manager to load data from
    /// </param>
    static void LoadContent(D3D11_Core& graphics,ContentManager& content);
    
    /// <summary>
    /// Allows the particle system type to unload any content that it
    /// previously loaded.
    /// <remarks>
    /// This should only be called after all instances of ParticleSystem are
    /// destroyed
    /// </remarks>
    /// </summary>
    /// <param name="graphics">
    /// graphics device that will be used by particle system instances
    /// </param>
    /// <param name="content">
    /// content manager to load data from
    /// </param>
    static void UnloadContent(D3D11_Core& graphics,ContentManager& content);
    
    /// <summary>
    /// Creates particle sysetm.  This should only be called after LoadContent
    /// has been done for the type.
    /// </summary>
    /// <param name="graphics">
    /// graphics device that this particle system will render to.  Must be the
    /// same as the one given to LoadContent
    /// </param>
    /// <param name="content">
    /// content manager to load data from
    /// </param>
    ParticleSystem(D3D11_Core& graphics,ContentManager& content);
    
    ~ParticleSystem();
    
    /// <summary>
    /// Retrieves the current emitter settings
    /// <remarks>
    /// Only for ease of network messages setting particle system values is the
    /// return type a reference instead of a const reference (or this function
    /// existing at all)
    /// </remarks>
    /// </summry>
    /// <returns>
    /// current emitter settings
    /// </returns>
    EmitterSettings& GetEmitterSettings();
    
    /// <summary>
    /// Retrieves the current particle settings
    /// <remarks>
    /// Only for ease of network messages setting particle system values is the
    /// return type a reference instead of a const reference (or this function
    /// existing at all)
    /// </remarks>
    /// </summry>
    /// <returns>
    /// current particle settings
    /// </returns>
    ParticleSettings& GetParticleSettings();
    
    /// <summary>
    /// Retrieves the coloring settings
    /// <remarks>
    /// Only for ease of network messages setting particle system values is the
    /// return type a reference instead of a const reference (or this function
    /// existing at all)
    /// </remarks>
    /// </summry>
    /// <returns>
    /// coloring settings
    /// </returns>
    ColoringSettings& GetColoringSettings();
    
    /// <summary>
    /// Sets the active particle dispersion
    /// </summary>
    /// <param name="disp">
    /// which particle dispersion should be active
    /// </param>
    void SetActiveDispersion(ParticleDispersion disp);
    
    /// <summary>
    /// Retrieves the dispersion settings
    /// </summary>
    /// <returns>
    /// reference to the various dispersion settings
    /// </returns>
    std::map<ParticleDispersion,DispersionBase*>& GetDispersions();
    
    /// <summary>
    /// Informs the particle system that the spawn rate or particle lifetime
    /// (the variables that affect the max number of particles existing at any
    /// given moment) have been updated
    /// </summary>
    /// <remarks>
    /// This is just to avoid having to calculate the max number of particles
    /// and comparing to the ring buffer size every update
    /// </remarks>
    /// <param name="mspf">
    /// milliseconds per frame
    /// </param>
    void UpdateMaxParticles(UINT mspf);
    
    /// <summary>
    /// Informs the particle system that particle shape has been changed
    /// </summary>
    void UpdateParticleShape();
    
    /// <summary>
    /// Updates the particle system based on time
    /// </summary>
    /// <param name="mss">
    /// Number of milliseconds to update the particle system by
    /// <remarks>
    /// It is pointless to call this function when this value is 0 aside from
    /// wasting performance
    /// </remarks>
    /// </param>
    /// <param name="context">
    /// D3D11 context to use for the update process
    /// </param>
    void Update(UINT ms,ID3D11DeviceContext* context);
    
    /// <summary>
    /// Draws the particle system
    /// </summary>
    /// <param name="cam">
    /// Camera to use for rendering the particle system
    /// </param>
    /// <param name="context">
    /// D3D11 context to draw to
    /// </param>
    void Draw(const Camera& cam,ID3D11DeviceContext* context);
    
  private:
    // disabled
    ParticleSystem();
    ParticleSystem(const ParticleSystem& cpy);
    ParticleSystem& operator=(const ParticleSystem& cpy);
    
    /// <summary>
    /// struct for the effect's constant buffer
    /// </summary>
    struct EffectConstBuffer
    {
      XMMATRIX vp;
      XMFLOAT4 cam_up;
      XMFLOAT4 cam_right;
      float    scale;
      int      use_texture;
    };
    
    /// <summary>
    /// Struct for containing the instance rendering data for a particle
    /// </summary>
    struct ParticleInstance
    {
      /// <summary>
      /// Position of the particle in world space
      /// </summary>
      XMFLOAT4 position;
      
      /// <summary>
      /// RGB color for the particle
      /// </summary>
      XMFLOAT3 color;
    };
    
    /// <summary>
    /// Struct for containing the non-rendering data for a particle
    /// </summary>
    struct ParticleData
    {
      /// <summary>
      /// position of the particle in world space
      /// </summary>
      XMFLOAT4 position;
      
      /// <summary>
      /// velocity vector in world space
      /// </summary>
      XMFLOAT4 velocity;
      
      /// <summary>
      /// how long, in ms, the particle has existed for
      /// </summary>
      UINT existed_for;
      
      /// <summary>
      /// color of the particle
      /// </summary>
      XMFLOAT3 color;
    };
    
    /// <summary>
    /// Creates new particles
    /// </summary>
    /// <param name="ms">
    /// amount of time, in milliseconds, to use for calculating the number of
    /// particles to create
    /// </param>
    void CreateParticles(UINT ms);
    
    /// <summary>
    /// Updates existing particles
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds to update the particles by
    /// </param>
    void UpdateParticles(UINT ms);
    
    /// <summary>
    /// Updates the instance buffer based on the current particle data
    /// </summary>
    /// <param name="context">
    /// D3D11 context to update the buffer in
    /// </param>
    void UpdateInstanceBuffer(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Settings for the emitter
    /// </summary>
    EmitterSettings m_emitter;
    
    /// <summary>
    /// Map of particle dispersion type to dispersion control instance
    /// </summary>
    /// <remarks>
    /// a map is used instead of a single instance so that when the editor
    /// switches dispersion types it doesn't need to re-send all the data
    /// associated with that type
    /// </remarks>
    std::map<ParticleDispersion,DispersionBase*> m_dispersions;
    
    /// <summary>
    /// ID of which particle dispersion control is the currently active one
    /// </summary>
    ParticleDispersion m_active_dispersion;
    
    /// <summary>
    /// Settings for the particles
    /// </summary>
    ParticleSettings m_particle_settings;
    
    /// <summary>
    /// Coloring settings for the particles
    /// </summary>
    ColoringSettings m_coloring_settings;
    
    /// <summary>
    /// vertex buffer for the instance data
    /// </summary>
    InstanceBuffer* m_instance;
    
    /// <summary>
    /// vertex buffer bindings for the particle geomerty and instance data
    /// </summary>
    VertexBufferArray* m_binding;
    
    /// <summary>
    /// graphics device to render to
    /// </summary>
    D3D11_Core& m_graphics;
    
    /// <summary>
    /// content manager that stuff was loaded from (needed by the destructor
    /// for unloading anything that was loaded)
    /// </summary>
    ContentManager& m_content;
    
    /// <summary>
    /// data about the particles that the particle instance renderer doesn't
    /// need
    /// </summary>
    RingBuffer<ParticleData>* m_particles;
    
    /// <summary>
    /// graphical data for each particle that is alive
    /// </summary>
    /// <remarks>
    /// this being a member variable is to avoid repeated allocation during
    /// Update.  Instead this is allocated to the max number of particles that
    /// can be alive at any given time and only the first m_alive contain the
    /// needed data.
    /// </remarks>
    ParticleInstance* m_instance_data;
    
    /// <summary>
    /// Time, in milliseconds, to add to the amount of time for spawning
    /// particles
    /// <remarks>
    /// This allows for spawning particles at a rate less than the rate Update
    /// is called
    /// </remarks>
    /// </summary>
    UINT m_additonal_spawn_time;
    
    /// <summary>
    /// keeps track of how long, in milliseconds, the emitter has been
    /// producing particles for
    /// </summary>
    UINT m_runtime;

    /**************************************************************************
     * Static_vars
     *************************************************************************/
    /// <summary>
    /// vertex buffer for triangle particles
    /// </summary>
    static VertexBuffer_PositionTexture* s_vert_tri;
    
    /// <summary>
    /// vertex buffer for square particles
    /// </summary>
    static VertexBuffer_PositionTexture* s_vert_square;
    
    /// <summary>
    /// index buffer for triangle particles
    /// </summary>
    static IndexBuffer16* s_index_tri;
    
    /// <summary>
    /// index buffer for square particles
    /// </summary>
    static IndexBuffer16* s_index_square;
    
    /// <summary>
    /// Instanced renderer for the particles
    /// </summary>
    static VSPSEffect* s_billboard_effect;
    
    /// <summary>
    /// texture sampler to use
    /// </summary>
    static TextureSampler* s_sampler;
    
    /// <summary>
    /// blend state for alpha blending
    /// </summary>
    static BlendState* s_alpha_blend;
    
    /// <summary>
    /// depth stencil state for disabling depth testing
    /// </summary>
    static DepthStencilState* s_no_depth_test;
};

#endif /* PARTICLE_SYSTEM_H */
