﻿#ifndef RING_BUFFER_H
#define RING_BUFFER_H

#include <vector>

template <class T>
class RingBuffer
{
  public:
    /// <summary>
    /// Creates an empty ring buffer with the specified number of elements
    /// </summary>
    /// <param name="length">
    /// number of elements in the ring buffer
    /// </param>
    RingBuffer(int length)
    :m_in_use(0),
     m_start(0),
     m_end(0),
     m_size(length)
    {
      m_data.reserve(length);
      for (int i = 0; i < length; ++i)
      {
        T tmp;
        m_data.push_back(tmp);
      }
    }
    
    /// <summary>
    /// Creates a ring buffer that initially has active elements of cpy.
    /// <remarks>
    /// If length is less than the length of cpy, then only the first length
    /// elements are copied
    /// </remarks>
    /// </summary>
    /// <param name="length">
    /// length of the new ring buffer
    /// </param>
    /// <param name="cpy">
    /// ring buffer to copy data from
    /// </param>
    RingBuffer(int length,const RingBuffer<T>& cpy)
    :m_in_use(0),
     m_start(0),
     m_end(0),
     m_size(length)
    {
      m_data.reserve(length);
      
      int cnt = (length < cpy.m_in_use) ? length : cpy.m_in_use;
      for (int i = 0; i < cnt; i++)
      {
        m_data.push_back(cpy.m_data[cpy.ToArrayIndex(i)]);
      }
      for (int i = cnt; i < length; ++i)
      {
        T tmp;
        m_data.push_back(tmp);
      }
    }
    
    ~RingBuffer()
    {
    }
    
    /// <summary>
    /// Adds the specified element to the ring buffer
    /// <remarks>
    /// This does not check for the buffer already being full since this
    /// particular implementation is intended for the particle system to keep
    /// track of a number of particles that was given to the constructor
    /// </remarks>
    /// </summary>
    /// <param name="val">
    /// value to add
    /// </param>
    void Add(T val)
    {
      // store the data
      m_data[m_end] = val;
      
      // update the end index
      m_end = (m_end + 1) % m_data.size();

      // update the number of elements in use
      ++m_in_use;
    }
    
    /// <summary>
    /// Removes the first cnt elements starting at Start's index
    /// <remarks>
    /// This does not check that there are at least cnt elements in use
    /// </remarks>
    /// </summary>
    /// <param name="cnt">
    /// Number of elements to remove from the ring buffer
    /// </param>
    void Free(int cnt)
    {
      m_start = (m_start + cnt) % m_data.size();
      m_in_use -= cnt;
      
      // note: the particle system is using this class on structs, so there is
      // no need to null out the entry in the ring buffer
    }
    
    /// <summary>
    /// Retrieves the max number of entries in the buffer
    /// </summary>
    /// <returns>
    /// max number of entries in the buffer
    /// </returns>
    int Size() const
    {
      return m_size;
    }
    
    /// <summary>
    /// Retrieves the number of entries currently in use
    /// </summary>
    /// <returns>
    /// number of entries in use
    /// </returns>
    int GetInUse() const
    {
      return m_in_use;
    }
    
    /// <summary>
    /// Retrieves the requested element
    /// <remarks>
    /// for performance reasons, this does not perform bounds checking
    /// </remarks>
    /// </summary>
    /// <param name="index">
    /// index of the requested element, in the range [0,GetInUse())
    /// </param>
    /// <returns>
    /// requested element
    /// </returns>
    const T& operator[](int index) const
    {
      return m_data[ToArrayIndex(index)];
    }
    
    /// <summary>
    /// Retrieves the requested element
    /// <remarks>
    /// for performance reasons, this does not perform bounds checking
    /// </remarks>
    /// </summary>
    /// <param name="index">
    /// index of the requested element, in the range [0,GetInUse())
    /// </param>
    /// <returns>
    /// requested element
    /// </returns>
    T& operator[](int index)
    {
      return m_data[ToArrayIndex(index)];
    }
    
  private:
    // disabled
    RingBuffer();
    RingBuffer(const RingBuffer& cpy);
    RingBuffer& operator=(const RingBuffer& cpy);
    
    /// <summary>
    /// Converts a ring buffer index to an index into the array Data
    /// </summary>
    /// <remarks>
    /// This is needed due to the particle system using structs as the type of
    /// the elements otherwise an enumerator could be used to update the values
    /// of the elements
    /// </remarks>
    /// <param name="i">
    /// ring buffer index
    /// </param>
    /// <returns>
    /// index into the Data array
    /// </returns>
    int ToArrayIndex(int i) const
    {
      return (m_start + i) % m_data.size();
    }
    
    /// <summary>
    /// storage of elements in the ring buffer
    /// </summary>
    std::vector<T> m_data;
    
    /// <summary>
    /// number of elements in Data that are in use
    /// </summary>
    int m_in_use;
    
    /// <summary>
    /// index of the first element in Data that is in use
    /// <remarks>
    /// Should only be used if InUse is > 0
    /// </remarks>
    /// </summary>
    int m_start;
    
    /// <summary>
    /// Index of the last elementin Data that is in use
    /// <remarks>
    /// Should only be used if InUse is > 0
    /// </remarks>
    /// </summary>
    int m_end;
    
    /// <summary>
    /// number of entries in the buffer
    /// <remarks>
    /// m_data.capacity is not used due to potential increases beyond the
    /// initially requested amount
    /// </remarks>
    /// </summary>
    int m_size;
};

#endif /* RING_BUFFER_H */
