#ifndef TCPSOCKET_H
#define TCPSOCKET_H

#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdint.h>

/// <summary>
/// TCP socket for communication
/// </summary>
class TCPSocket
{
  public:
    TCPSocket(SOCKET sock);
    ~TCPSocket();
    
    /// <summary>
    /// Checks whether there is at least 1 byte ready for reading or not
    /// </summary>
    /// <returns>
    /// true  if there is at least 1 byte ready for reading
    /// false otherwise
    /// </returns>
    bool DataPending();
    
    /// <summary>
    /// Reads at most the requested number of bytes from the socket
    /// </summary>
    /// <param name="buffer">
    /// where to place the read data.  Only elements for read values are
    /// modified
    /// </param>
    /// <param name="count">
    /// number of bytes to read
    /// <remarks>
    /// It is up to the caller to make sure that buffer is at least
    /// count + offset bytes long
    /// </remarks>
    /// </param>
    /// <param name="offset">
    /// first index into buffer to start placing read bytes
    /// </param>
    /// <returns>
    /// number of bytes read
    /// 0 if the socket is closed
    /// -1 on error
    /// </returns>
    int Read(char* buffer,int count,int offset);
    
    /// <summary>
    /// Reads a int32_t from the socket, leaving it in the received byte order
    /// </summary>
    /// <param name="val">
    /// where to put the read value
    /// </param>
    /// <returns>
    /// number of bytes read
    /// 0 if the socket is closed
    /// -1 on error
    /// </returns>
    int ReadINT32N(int32_t& val);
    
    /// <summary>
    /// Reads a int32_t from the socket, converting it from network to host
    /// byte order
    /// </summary>
    /// <param name="val">
    /// where to put the read value
    /// </param>
    /// <returns>
    /// number of bytes read
    /// 0 if the socket is closed
    /// -1 on error
    /// </returns>
    int ReadINT32H(int32_t& val);
    
    /// <summary>
    /// Reads a uint32_t from the socket, leaving it in the received byte order
    /// </summary>
    /// <param name="val">
    /// where to put the read value
    /// </param>
    /// <returns>
    /// number of bytes read
    /// 0 if the socket is closed
    /// -1 on error
    /// </returns>
    int ReadUINT32N(uint32_t& val);
    
    /// <summary>
    /// Reads a uint32_t from the socket, converting it from network to host
    /// byte order
    /// </summary>
    /// <param name="val">
    /// where to put the read value
    /// </param>
    /// <returns>
    /// number of bytes read
    /// 0 if the socket is closed
    /// -1 on error
    /// </returns>
    int ReadUINT32H(uint32_t& val);
    
    /// <summary>
    /// Sends the data in the buffer across the socket
    /// </summary>
    /// <param name="buffer">
    /// data to send
    /// </param>
    /// <param name="count">
    /// number of bytes to send
    /// <remarks>
    /// It is up to the caller to make sure that buffer is at least
    /// count + offset bytes long
    /// </remarks>
    /// </param>
    /// <param name="offset">
    /// first index into buffer to start placing writing bytes
    /// </param>
    /// <returns>
    /// number of bytes sent
    /// -1 on error
    /// </returns>
    int Send(const char* buffer,int count,int offset);
    
  private:
    // disabled
    TCPSocket();
    TCPSocket(const TCPSocket& cpy);
    TCPSocket& operator=(const TCPSocket& cpy);
    
    /// <summary>
    /// TCP socket to use for communication
    /// </summary>
    SOCKET m_sock;
};

#endif /* TCPSOCKET_H */
